/*
 * This file maps all the IFI I/O port names
 * to Team 1511 robot input/output functional
 * names. This makes it easy to handle pin reallocations
 * and remember variable names.
 *
 * These should be kept in sync with the rc and oi Excel
 * spreadsheet mappings of the RC and OI pinouts. Changes
 * need to be coordinated with Electrical.
 */

#ifndef IOMAP_H
#define IOMAP_H

#include "ifi_default.h"
#include "ifi_aliases.h"


/*
 * PWM Aliases.
 * 7 - 10, 14 - 16 are currently unused
 * 13 is dedicated to providing power for the camera
 */
#define pwm_left_drive                 pwm01
#define pwm_right_drive                pwm02
#define pwm_turret                     pwm03
#define pwm_arm                        pwm04
#define pwm_wrist                      pwm05
#define pwm_manip                      pwm06

#define pwm_camera_pan                 pwm11
#define pwm_camera_tilt                pwm12



/*
 * Relay Aliases
 * None for now....
 * 1 - 8 Unused
 */


/* 
 * Analog inputs
 * 4 - 16 currently unused
 */
#define anain_turret                   rc_ana_in01
#define anain_arm                      rc_ana_in02
#define anain_wrist                    rc_ana_in03
#define anain_left_lo_pressure         rc_ana_in04
#define anain_right_lo_pressure        rc_ana_in05
#define anain_right_hi_pressure        rc_ana_in06
#define anain_left_hi_pressure         rc_ana_in07


/* Digital I/O
 * NOTE: If you change the direction of the pin, you
 *       must change the init code in 
 *       user_routines.c::User_Initialization()
 * 1, 2, 5-8, 12-18 are not in use
 */

// For the encoders, enable the correct one, and also 
//   alias the functions
#define digin_left_wheel_enc           rc_dig_in03
#define ENABLE_ENCODER_3
#define Get_Left_Encoder_Count         Get_Encoder_3_Count
#define Reset_Left_Encoder_Count       Reset_Encoder_3_Count

#define digin_right_wheel_enc          rc_dig_in04
#define ENABLE_ENCODER_4
#define Get_Right_Encoder_Count        Get_Encoder_4_Count
#define Reset_Right_Encoder_Count      Reset_Encoder_4_Count

/* These are active low */
#define digin_manip_limit_close        (!rc_dig_in09)
#define digin_manip_limit_open         (!rc_dig_in10)

#define digin_autoselect_b5            rc_dig_in13
#define digin_autoselect_b4            rc_dig_in14
#define digin_autoselect_b3            rc_dig_in15
#define digin_autoselect_b2            rc_dig_in16
#define digin_autoselect_b1            rc_dig_in17
#define digin_autoselect_b0            rc_dig_in18



/* 
 *OI Inputs
 */

#define oi_right_drive                 p2_x
#define oi_left_drive                  p2_y
#define oi_turbo_left                  p2_sw_trig
#define oi_turbo_right                 p2_sw_top
#define oi_slow_left                   p2_sw_aux1
#define oi_slow_right                  p2_sw_aux2

#define oi_arm                         p4_y
#define oi_turret                      p4_wheel
#define oi_wrist                       p4_aux
#define oi_manip_close                 p4_sw_trig
#define oi_manip_open                  p4_sw_top
#define oi_wrist_fine_up               p4_sw_aux1
#define oi_wrist_fine_down             p4_sw_aux2

#define oi_auto_wrist                  p3_sw_aux2

// In debug mode, these cause sensor calibration data to
// be printed.  In order, the sensors debugged are:
// manipulator, wrist, arm, turret, drive encoders
#define oi_armpos_high                 p1_sw_trig
#define oi_armpos_mid                  p1_sw_top
#define oi_armpos_low                  p1_sw_aux1
#define oi_armpos_floor                p1_sw_aux2
#define oi_turret_center               (p1_y > 200)
#define oi_park                        (p1_aux > 200)
#define oi_red_light                   (p1_x > 200)

#define oi_debug_mode                  (p3_aux > 200)

#define oi_working_turret              (p3_x > 200 && !oi_debug_mode)
#define oi_working_arm                 (p3_y > 200 && !oi_debug_mode)
#define oi_working_wrist               (p3_wheel > 200 && !oi_debug_mode)
#define oi_working_manip               (p3_sw_aux1 && !oi_debug_mode)

#define oi_led_autoarm                  Pwm1_green
#define oi_led_turret_centered          Pwm1_red
#define oi_led_manip_openclose          Pwm2_green
#define oi_led_tube_grabbed             Pwm2_red

#define oi_led_normal_mode              Relay1_green
#define oi_led_wrist_limit              Relay1_red
#define oi_led_turret_limit             Relay2_green
#define oi_led_arm_limit                Relay2_red

#endif

