
/*******************************************************************************
* FILE NAME: user_routines.c <FRC VERSION>
*
* DESCRIPTION:
*  This file contains the default mappings of inputs  
*  (like switches, joysticks, and buttons) to outputs on the RC.  
*
* USAGE:
*  You can either modify this file to fit your needs, or remove it from your 
*  project and replace it with a modified copy. 
*
*******************************************************************************/

/* This is jeff showing everyone how to use SVN */
#include <stdio.h>

#include "ifi_aliases.h"
#include "ifi_default.h"
#include "ifi_utilities.h"
#include "user_routines.h"
#include "serial_ports.h"
#include "camera.h"
#include "tracking.h"
#include "encoder.h"
#include "ifi_aliases.h"
#include "control.h"
#include "motion.h"



/* Prototypes */
static void Decode_Analog_Buttons(void);
static void Arm_Control(void);
static void Manip_Control(void);
static void Manual_Arm_Control(void);
static void Manual_Drive(void);
static void Red_Light_Control(void);
static void Send_Feedback(void);
static void Debug_Print(void);
static ArmCommand Decode_Arm_Auto(void);

//you lost the variable declarations that are global
char p1_button5;
char p2_button5;
char p3_button5;
char p4_button5;

char p1_button6;
char p2_button6;
char p3_button6;
char p4_button6;

char p1_button7;
char p2_button7;
char p3_button7;
char p4_button7;

char p1_button8;
char p2_button8;
char p3_button8;
char p4_button8;



typedef enum {
  STATUS_WORKING,
  STATUS_DONE,
  STATUS_QUIET
} ArmStatus;

static ArmStatus arm_auto_status;
static ArmCommand arm_auto_lastcmd;


// Drive code LUTs
static const rom unsigned char ten_pct[];
static const rom unsigned char drive_lut[];
static const rom unsigned char turbo_drive_lut[];
static const rom unsigned char slow_drive_lut[];


/*******************************************************************************
* FUNCTION NAME: User_Initialization
* PURPOSE:       This routine is called first (and only once) in the Main function.  
*                You may modify and add to this function.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void User_Initialization (void)
{
  Set_Number_of_Analog_Channels(SIXTEEN_ANALOG);    /* DO NOT CHANGE! */

/* FIRST: Set up the I/O pins you want to use as digital INPUTS. */
  digital_io_01 = digital_io_02 = digital_io_03 = digital_io_04 = INPUT;
  digital_io_05 = digital_io_06 = digital_io_07 = digital_io_08 = INPUT;
  digital_io_09 = digital_io_10 = digital_io_11 = digital_io_12 = INPUT;
  digital_io_13 = digital_io_14 = digital_io_15 = digital_io_16 = INPUT;
  digital_io_17 = digital_io_18 = INPUT;  /* Used for pneumatic pressure switch. */


  /* Set your PWM output types for PWM OUTPUTS 13-16.
  /*   Choose from these parameters for PWM 13-16 respectively:               */
  /*     IFI_PWM  - Standard IFI PWM output generated with Generate_Pwms(...) */
  /*     USER_CCP - User can use PWM pin as digital I/O or CCP pin.           */
  Setup_PWM_Output_Type(IFI_PWM,IFI_PWM,IFI_PWM,IFI_PWM);

  /* 
     Example: The following would generate a 40KHz PWM with a 50% duty cycle on the CCP2 pin:

         CCP2CON = 0x3C;
         PR2 = 0xF9;
         CCPR2L = 0x7F;
         T2CON = 0;
         T2CONbits.TMR2ON = 1;

         Setup_PWM_Output_Type(USER_CCP,IFI_PWM,IFI_PWM,IFI_PWM);
  */

  // Initialize drive and arm control structures
  Motion_Init();
  Arm_Init();

  Initialize_Encoders();

  /* Set up kevin.org serial driver */
#if defined(ENABLE_SERIAL_PORT_ONE_RX) || defined(ENABLE_SERIAL_PORT_ONE_TX)
  Init_Serial_Port_One();
  stdout_serial_port = SERIAL_PORT_ONE;
#endif
#if defined(ENABLE_SERIAL_PORT_TWO_RX) || defined(ENABLE_SERIAL_PORT_TWO_TX)
  Init_Serial_Port_Two();
#endif

  // Set all PWMs to neutral; reset all state.
  Reset_All();


  /* Add any other initialization code here. */
 
  Putdata(&txdata);             /* DO NOT CHANGE! */

  printf("IFI 2006 User Processor Initialized ...\r");  /* Optional - Print initialization message. */

  User_Proc_Is_Ready();         /* DO NOT CHANGE! - last line of User_Initialization */
}


/*
 * Resets all PWMs, Relays, and user_routines state.
 * Shared across initial initialization, enter, and exit of
 * autonomous.
 */
void Reset_All(void)
{
    // All PWMs to neutral
    pwm01 = pwm02 = pwm03 = pwm04 = pwm05 = pwm06 = pwm07 = pwm08 = 127;
    pwm09 = pwm10 = pwm11 = pwm12 = pwm13 = pwm14 = pwm15 = pwm16 = 127;
    
    // All Relays to neutral
    relay1_fwd = relay1_rev = relay2_fwd = relay2_rev = 0;
    relay3_fwd = relay3_rev = relay4_fwd = relay4_rev = 0;
    relay5_fwd = relay5_rev = relay6_fwd = relay6_rev = 0;
    relay7_fwd = relay7_rev = relay8_fwd = relay8_rev = 0;

    // Reset state
    Arm_Reset();
    arm_auto_status = STATUS_QUIET;
    arm_auto_lastcmd = ARM_CMD_NONE;
}



/*******************************************************************************
* FUNCTION NAME: Process_Data_From_Master_uP
* PURPOSE:       Executes every 26.2ms when it gets new data from the master 
*                microprocessor.
* CALLED FROM:   main.c
* ARGUMENTS:     none
* RETURNS:       void
*******************************************************************************/
void Process_Data_From_Master_uP(void)
{
  Getdata(&rxdata);   /* Get fresh data from the master microprocessor. */

  Camera_Handler();
  Servo_Track();
  Decode_Analog_Buttons();

  Manual_Drive();
  Manip_Control();
    

  // Update sensor broken-ness states 
  //  for arm control module. Broken-ness state is
  //  preserved for autonomous.
  Arm_Sensor_Set_Broken(
    (char)(!oi_working_turret),
    (char)(!oi_working_arm),
    (char)(!oi_working_wrist),
    (char)(!oi_working_manip)
  );

  if (oi_debug_mode) {
    // Debug takes over auto-arm buttons for debug functions
    // So just jump straight to manual arm controller
    Manual_Arm_Control();
    Debug_Print();
  } else {
    Arm_Control();
  }

  Red_Light_Control();

  Send_Feedback();

  //Generate_Pwms(pwm13,pwm14,pwm15,pwm16);

  Putdata(&txdata);             /* DO NOT CHANGE! */
}


static void Manip_Control(void)
{
    if (oi_manip_open) {
        Open_Manip();
    } else if (oi_manip_close) {
        Close_Manip();
    } else {
        Stop_Manip();
    }
}


static void Arm_Control(void)
{
    ArmCommand cur_auto = Decode_Arm_Auto();
  
    if (cur_auto != ARM_CMD_NONE) {
        if (cur_auto != arm_auto_lastcmd)
            Arm_Auto_Init(cur_auto);

        if (Arm_Auto_Process())
            arm_auto_status = STATUS_DONE;
        else
            arm_auto_status = STATUS_WORKING;

        arm_auto_lastcmd = cur_auto;
          
            
    } else {
        if (arm_auto_lastcmd == ARM_CMD_PARK) {
          if (Arm_Auto_Process())
              arm_auto_status = STATUS_DONE;
          else
              arm_auto_status = STATUS_WORKING;
          return;
        }

        Manual_Arm_Control();
    }

}


static void Manual_Arm_Control(void)
{
  if (oi_turret_center)
    Arm_Process(oi_arm, -1, oi_wrist);
  else if (oi_auto_wrist && oi_working_arm)
    Arm_Process(oi_arm, oi_turret, -1);
  else
  {
    // Map throttle value to wrist angle
    int angle = (int)((long)(WRIST_MAX - WRIST_MIN) * oi_wrist / 255) + WRIST_MIN;
    Arm_Process(oi_arm, oi_turret, angle);
  }
  arm_auto_lastcmd = ARM_CMD_NONE;
  arm_auto_status = STATUS_QUIET;
}


static void Manual_Drive(void) {
    if (oi_slow_left && oi_turbo_left && oi_slow_right && oi_turbo_right)
    {
        pwm_right_drive = 255 - ten_pct[turbo_drive_lut[oi_left_drive]];
        pwm_left_drive = 255 - turbo_drive_lut[oi_right_drive];
    }
    else
    {
      if (oi_slow_left)
          pwm_left_drive = slow_drive_lut[oi_left_drive];
      else if (oi_turbo_left)
          pwm_left_drive = turbo_drive_lut[oi_left_drive];
      else
          pwm_left_drive = drive_lut[oi_left_drive];
  	
      if (oi_slow_right)
          pwm_right_drive = ten_pct[slow_drive_lut[oi_right_drive]];
      else if (oi_turbo_right)
          pwm_right_drive = ten_pct[turbo_drive_lut[oi_right_drive]];
      else
          pwm_right_drive = ten_pct[drive_lut[oi_right_drive]];
    }
}

static void Red_Light_Control(void)
{
    relay1_fwd = oi_red_light;
}


static void Send_Feedback(void)
{
    static int blink_count = 0;

    // Camera feedback to dashboard port
    User_Byte3 = GT_Packet_Data.x1;
    User_Byte4 = GT_Packet_Data.y1;
    User_Byte5 = GT_Packet_Data.x2;
    User_Byte6 = GT_Packet_Data.y2;

    if (user_display_mode) {
        User_Mode_byte = backup_voltage*10;

    } else {
        // LED feedback
        oi_led_normal_mode = !oi_debug_mode;
        oi_led_wrist_limit = 
            oi_working_wrist && Wrist_At_Limit();
        oi_led_arm_limit = 
            oi_working_arm && Arm_At_Limit();
        oi_led_turret_limit = 
            oi_working_turret && Turret_At_Limit();
        oi_led_turret_centered = 
            oi_working_turret && Turret_Centered();

        switch (Get_Manip_State()) {
            case 0:
            case 1:
                oi_led_manip_openclose = 1;
                oi_led_tube_grabbed = 0;
                break;
            case 2:
                oi_led_manip_openclose = 1;
                oi_led_tube_grabbed = 1;
                break;
            case 3:
            default:
                oi_led_manip_openclose = 0;
                oi_led_tube_grabbed = 0;
                break;
        }

        switch (arm_auto_status) {
            case STATUS_DONE:
                oi_led_autoarm = 1;
                break;
            case STATUS_WORKING:
                if (blink_count++ == 25) {
                    blink_count = 0;
                    oi_led_autoarm ^= 1;
                }
                break;
            case STATUS_QUIET:
                oi_led_autoarm = 0;
                break;
        }
    }
}



static ArmCommand Decode_Arm_Auto(void)
{
    ArmCommand ret;

    if (oi_armpos_high) {
        ret = ARM_CMD_HI;
    } else if (oi_armpos_mid) {
        ret = ARM_CMD_MID;
    } else if (oi_armpos_low) {
        ret = ARM_CMD_LOW;
    } else if (oi_armpos_floor) {
        ret = ARM_CMD_PICKUP;
    } else if (oi_park) {
        ret = ARM_CMD_PARK;
    } else {
        ret = ARM_CMD_NONE;
    }

    return ret;
}



static void Debug_Print(void)
{
    static int pcount = 0;

    if (pcount++ < 30)
        return;

    pcount = 0;

    if (oi_armpos_high) {
        printf("Manip ---\r\n");
        printf("Left Hi Pressure: %4d\r\n", Get_Analog_Value(anain_left_hi_pressure));
        printf("Right Hi Pressure: %4d\r\n", Get_Analog_Value(anain_right_hi_pressure));
        printf("Left Lo Pressure: %4d\r\n", Get_Analog_Value(anain_left_lo_pressure));
        printf("Right Lo Pressure: %4d\r\n", Get_Analog_Value(anain_right_lo_pressure));
        printf("Open  Limit:    %d\r\n", digin_manip_limit_open);
        printf("Close Limit:    %d\r\n", digin_manip_limit_close);
        printf("\r\n");
    }

    if (oi_armpos_mid) {
        printf("Wrist POT: %4d\r\n", Get_Analog_Value(anain_wrist));
        printf("\r\n");
    }

    if (oi_armpos_low) {
        printf("Arm POT: %4d\r\n", Get_Analog_Value(anain_arm));
        printf("\r\n");
    }

    if (oi_armpos_floor) {
        printf("Turret POT: %4d\r\n", Get_Analog_Value(anain_turret));
        printf("\r\n");
    }

    if (oi_turret_center) {
        printf("Autonomous selection: %4d\r\n", Get_Autonomous_Selection()); 
        printf("Autonomous bits: %d %d %d %d %d %d\r\n", digin_autoselect_b5, digin_autoselect_b4, digin_autoselect_b3, digin_autoselect_b2, digin_autoselect_b1, digin_autoselect_b0); 
        printf("\r\n");
    }
    
    if (oi_park) {
        long Encoder_Count;
      
      	Encoder_Count = Get_Right_Encoder_Count();
      	printf("Right Enc=%d\r\n",(int)Encoder_Count);
      
      	Encoder_Count = Get_Left_Encoder_Count();
      	printf("Left  Enc=%d\r\n",(int)Encoder_Count);
      
      	printf("\r\n");
    }
}




static void Decode_Analog_Buttons(void)
{
    static int i=0;
    p1_button5	= AuxAnalogTerp (5,p1_wheel);
    p2_button5	= AuxAnalogTerp (5,p2_wheel);
    p3_button5	= AuxAnalogTerp (5,p3_wheel);
    p4_button5	= AuxAnalogTerp (5,p4_wheel);
    
    p1_button6	= AuxAnalogTerp (6,p1_wheel);
    p2_button6	= AuxAnalogTerp (6,p2_wheel);
    p3_button6	= AuxAnalogTerp (6,p3_wheel);
    p4_button6	= AuxAnalogTerp (6,p4_wheel);
    
    p1_button7	= AuxAnalogTerp (7,p1_wheel);
    p2_button7	= AuxAnalogTerp (7,p2_wheel);
    p3_button7	= AuxAnalogTerp (7,p3_wheel);
    p4_button7	= AuxAnalogTerp (7,p4_wheel);
    
    p1_button8	= AuxAnalogTerp (8,p1_wheel);
    p2_button8	= AuxAnalogTerp (8,p2_wheel);
    p3_button8	= AuxAnalogTerp (8,p3_wheel);
    p4_button8	= AuxAnalogTerp (8,p4_wheel);
    
    
    /*
    i++;
    if (i > 29)
    {
        printf("p1_button5 = %d\r", p1_button5);
        printf("p1_button6 = %d\r", p1_button6);
        printf("p1_button7 = %d\r", p1_button7);
        printf("p1_button8 = %d\r", p1_button8);
        i=0;
    } 
    */   
}


/*
* Wrote by Jim Bossert
* it tralation the analog infomation from aux 
* analog from controller in ditigal infomation
* used for addiation buttons(above 4)
*/
char AuxAnalogTerp (int WhatButton, int port_analog)
{
	char Bit7, Bit6, Bit5, Bit4;
	char answer;

	//bitwise AND- compare auxAnalog with a specific mask
	//	to check if a bit is set
	//If you want to check bit 0 (lowest)
	// Bit0=auxAnalog & (00000001);

	
	Bit7=(~((port_analog & 128)>>7))+2;	//button 8 is highest bit
	Bit6=(~((port_analog & 64)>>6))+2;	//button 7 is 6 bit
	Bit5=(~((port_analog & 32)>>5))+2;	//button 6 is 5 bit
	Bit4=(~((port_analog & 16)>>4))+2;	//button 5 is 4 bit
	

	if (WhatButton == 5)
		answer=Bit4;
	else if (WhatButton == 6)
		answer=Bit5;
	else if (WhatButton == 7)
		answer=Bit6;
	else if (WhatButton == 8)
		answer=Bit7;
	else 
		answer=0;

	return answer;
}

// Drive lookup tables
// For each function, xmin = -127 and xmax = 127

//(x^3) * (55/2048383) + 127
static const rom unsigned char drive_lut [] = {
	72, 73, 75, 76, 77, 78, 79, 81, 
	82, 83, 84, 85, 86, 87, 88, 89, 
	90, 91, 92, 93, 94, 95, 96, 97, 
	98, 99, 99, 100, 101, 102, 102, 103, 
	104, 105, 105, 106, 107, 107, 108, 109, 
	109, 110, 111, 111, 112, 112, 113, 113, 
	114, 114, 115, 115, 116, 116, 117, 117, 
	117, 118, 118, 119, 119, 119, 120, 120, 
	120, 121, 121, 121, 121, 122, 122, 122, 
	123, 123, 123, 123, 123, 124, 124, 124, 
	124, 124, 125, 125, 125, 125, 125, 125, 
	125, 126, 126, 126, 126, 126, 126, 126, 
	126, 126, 126, 126, 126, 127, 127, 127, 
	127, 127, 127, 127, 127, 127, 127, 127, 
	127, 127, 127, 127, 127, 127, 127, 127, 
	127, 127, 127, 127, 127, 127, 127, 127, 
	127, 127, 127, 127, 127, 127, 127, 127, 
	127, 127, 127, 127, 127, 127, 127, 127, 
	127, 127, 127, 127, 127, 127, 127, 127, 
	127, 127, 128, 128, 128, 128, 128, 128, 
	128, 128, 128, 128, 128, 128, 129, 129, 
	129, 129, 129, 129, 129, 130, 130, 130, 
	130, 130, 131, 131, 131, 131, 131, 132, 
	132, 132, 133, 133, 133, 133, 134, 134, 
	134, 135, 135, 135, 136, 136, 137, 137, 
	137, 138, 138, 139, 139, 140, 140, 141, 
	141, 142, 142, 143, 143, 144, 145, 145, 
	146, 147, 147, 148, 149, 149, 150, 151, 
	152, 152, 153, 154, 155, 155, 156, 157, 
	158, 159, 160, 161, 162, 163, 164, 165, 
	166, 167, 168, 169, 170, 171, 172, 173, 
	175, 176, 177, 178, 179, 181, 182, 183
};

static const rom unsigned char ten_pct [] = {
	13, 14, 15, 15, 16, 17, 18, 19, 
	20, 21, 22, 23, 24, 24, 25, 26, 
	27, 28, 29, 30, 31, 32, 33, 33, 
	34, 35, 36, 37, 38, 39, 40, 41, 
	42, 42, 43, 44, 45, 46, 47, 48, 
	49, 50, 51, 51, 52, 53, 54, 55, 
	56, 57, 58, 59, 60, 60, 61, 62, 
	63, 64, 65, 66, 67, 68, 69, 69, 
	70, 71, 72, 73, 74, 75, 76, 77, 
	78, 78, 79, 80, 81, 82, 83, 84, 
	85, 86, 87, 87, 88, 89, 90, 91, 
	92, 93, 94, 95, 96, 96, 97, 98, 
	99, 100, 101, 102, 103, 104, 105, 105, 
	106, 107, 108, 109, 110, 111, 112, 113, 
	114, 114, 115, 116, 117, 118, 119, 120, 
	121, 122, 123, 123, 124, 125, 126, 127, 
	128, 129, 130, 131, 132, 132, 133, 134, 
	135, 136, 137, 138, 139, 140, 141, 141, 
	142, 143, 144, 145, 146, 147, 148, 149, 
	150, 150, 151, 152, 153, 154, 155, 156, 
	157, 158, 159, 159, 160, 161, 162, 163, 
	164, 165, 166, 167, 168, 168, 169, 170, 
	171, 172, 173, 174, 175, 176, 177, 177, 
	178, 179, 180, 181, 182, 183, 184, 185, 
	186, 186, 187, 188, 189, 190, 191, 192, 
	193, 194, 195, 195, 196, 197, 198, 199, 
	200, 201, 202, 203, 204, 204, 205, 206, 
	207, 208, 209, 210, 211, 212, 213, 213, 
	214, 215, 216, 217, 218, 219, 220, 221, 
	222, 222, 223, 224, 225, 226, 227, 228, 
	229, 230, 231, 231, 232, 233, 234, 235, 
	236, 237, 238, 239, 240, 240, 241, 242
};

//(x^3) * (127/2048383) + 127
static const rom unsigned char turbo_drive_lut[] = {
	0x00, 0x03, 0x06, 0x09, 0x0c, 0x0e, 0x11, 0x14, 
	0x17, 0x19, 0x1c, 0x1e, 0x21, 0x23, 0x26, 0x28, 
	0x2a, 0x2c, 0x2f, 0x31, 0x33, 0x35, 0x37, 0x39, 
	0x3b, 0x3d, 0x3f, 0x41, 0x43, 0x45, 0x46, 0x48, 
	0x4a, 0x4c, 0x4d, 0x4f, 0x50, 0x52, 0x53, 0x55, 
	0x56, 0x58, 0x59, 0x5a, 0x5c, 0x5d, 0x5e, 0x5f, 
	0x60, 0x62, 0x63, 0x64, 0x65, 0x66, 0x67, 0x68, 
	0x69, 0x6a, 0x6b, 0x6c, 0x6c, 0x6d, 0x6e, 0x6f, 
	0x6f, 0x70, 0x71, 0x72, 0x72, 0x73, 0x74, 0x74, 
	0x75, 0x75, 0x76, 0x76, 0x77, 0x77, 0x78, 0x78, 
	0x79, 0x79, 0x79, 0x7a, 0x7a, 0x7a, 0x7b, 0x7b, 
	0x7b, 0x7c, 0x7c, 0x7c, 0x7c, 0x7d, 0x7d, 0x7d, 
	0x7d, 0x7d, 0x7d, 0x7e, 0x7e, 0x7e, 0x7e, 0x7e, 
	0x7e, 0x7e, 0x7e, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 
	0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 
	0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 
	0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 
	0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 
	0x7f, 0x7f, 0x7f, 0x7f, 0x80, 0x80, 0x80, 0x80, 
	0x80, 0x80, 0x80, 0x80, 0x81, 0x81, 0x81, 0x81, 
	0x81, 0x81, 0x82, 0x82, 0x82, 0x82, 0x83, 0x83, 
	0x83, 0x84, 0x84, 0x84, 0x85, 0x85, 0x85, 0x86, 
	0x86, 0x87, 0x87, 0x88, 0x88, 0x89, 0x89, 0x8a, 
	0x8a, 0x8b, 0x8c, 0x8c, 0x8d, 0x8e, 0x8f, 0x8f, 
	0x90, 0x91, 0x92, 0x92, 0x93, 0x94, 0x95, 0x96, 
	0x97, 0x98, 0x99, 0x9a, 0x9b, 0x9c, 0x9e, 0x9f, 
	0xa0, 0xa1, 0xa2, 0xa4, 0xa5, 0xa6, 0xa8, 0xa9, 
	0xab, 0xac, 0xae, 0xaf, 0xb1, 0xb2, 0xb4, 0xb6, 
	0xb8, 0xb9, 0xbb, 0xbd, 0xbf, 0xc1, 0xc3, 0xc5, 
	0xc7, 0xc9, 0xcb, 0xcd, 0xcf, 0xd2, 0xd4, 0xd6, 
	0xd8, 0xdb, 0xdd, 0xe0, 0xe2, 0xe5, 0xe7, 0xea, 
	0xed, 0xf0, 0xf2, 0xf5, 0xf8, 0xfb, 0xff, 0xff
};

// (x^3) * (35/2048383) + 127
static const rom unsigned char slow_drive_lut [] = {
	0x5c, 0x5d, 0x5e, 0x5e, 0x5f, 0x60, 0x61, 0x61, 
	0x62, 0x63, 0x64, 0x64, 0x65, 0x66, 0x66, 0x67, 
	0x68, 0x68, 0x69, 0x69, 0x6a, 0x6b, 0x6b, 0x6c, 
	0x6c, 0x6d, 0x6d, 0x6e, 0x6e, 0x6f, 0x6f, 0x70, 
	0x70, 0x71, 0x71, 0x72, 0x72, 0x73, 0x73, 0x73, 
	0x74, 0x74, 0x75, 0x75, 0x75, 0x76, 0x76, 0x76, 
	0x77, 0x77, 0x77, 0x77, 0x78, 0x78, 0x78, 0x79, 
	0x79, 0x79, 0x79, 0x7a, 0x7a, 0x7a, 0x7a, 0x7b, 
	0x7b, 0x7b, 0x7b, 0x7b, 0x7b, 0x7c, 0x7c, 0x7c, 
	0x7c, 0x7c, 0x7c, 0x7d, 0x7d, 0x7d, 0x7d, 0x7d, 
	0x7d, 0x7d, 0x7d, 0x7e, 0x7e, 0x7e, 0x7e, 0x7e, 
	0x7e, 0x7e, 0x7e, 0x7e, 0x7e, 0x7e, 0x7e, 0x7e, 
	0x7e, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 
	0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 
	0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 
	0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 
	0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 
	0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 
	0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 
	0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x7f, 0x80, 0x80, 
	0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 0x80, 
	0x80, 0x80, 0x80, 0x80, 0x81, 0x81, 0x81, 0x81, 
	0x81, 0x81, 0x81, 0x81, 0x82, 0x82, 0x82, 0x82, 
	0x82, 0x82, 0x83, 0x83, 0x83, 0x83, 0x83, 0x83, 
	0x84, 0x84, 0x84, 0x84, 0x85, 0x85, 0x85, 0x85, 
	0x86, 0x86, 0x86, 0x87, 0x87, 0x87, 0x87, 0x88, 
	0x88, 0x88, 0x89, 0x89, 0x89, 0x8a, 0x8a, 0x8b, 
	0x8b, 0x8b, 0x8c, 0x8c, 0x8d, 0x8d, 0x8e, 0x8e, 
	0x8f, 0x8f, 0x90, 0x90, 0x91, 0x91, 0x92, 0x92, 
	0x93, 0x93, 0x94, 0x95, 0x95, 0x96, 0x96, 0x97, 
	0x98, 0x98, 0x99, 0x9a, 0x9a, 0x9b, 0x9c, 0x9d, 
	0x9d, 0x9e, 0x9f, 0xa0, 0xa0, 0xa1, 0xa2, 0xa2
};

/******************************************************************************/
/******************************************************************************/
/******************************************************************************/
